"""The Parade data processing engine and services."""

from __future__ import absolute_import, division, print_function, with_statement

# version is a human-readable version number.
version = "0.1.0"

# version_info is a four-tuple for programmatic comparison. The first
# three numbers are the components of the version number.  The fourth
# is zero for an official release, positive for a development branch,
# or negative for a release candidate or beta (after the base version
# number has been incremented)
version_info = (0, 1, 0, -1)
